// Copyright 2023 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   extensions/common/api/app_runtime.idl
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "extensions/common/api/app_runtime.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"

using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace app_runtime {
//
// Types
//

LaunchItem::Entry::Entry()
 {}

LaunchItem::Entry::~Entry() = default;
LaunchItem::Entry::Entry(Entry&& rhs) = default;
LaunchItem::Entry& LaunchItem::Entry::operator=(Entry&& rhs) = default;
// static
bool LaunchItem::Entry::Populate(
    const base::Value& value, Entry* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  out->additional_properties.Merge(dict.Clone());
  return true;
}

base::Value::Dict LaunchItem::Entry::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Merge(additional_properties.Clone());

  return to_value_result;
}



LaunchItem::LaunchItem()
 {}

LaunchItem::~LaunchItem() = default;
LaunchItem::LaunchItem(LaunchItem&& rhs) = default;
LaunchItem& LaunchItem::operator=(LaunchItem&& rhs) = default;
// static
bool LaunchItem::Populate(
    const base::Value& value, LaunchItem* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* entry_value = dict.Find("entry");
  if (!entry_value) {
    return false;
  }
  {
    if (!(*entry_value).is_dict()) {
      return false;
    }
    if (!Entry::Populate((*entry_value), &out->entry)) {
      return false;
    }
  }

  const base::Value* type_value = dict.Find("type");
  if (type_value) {
    {
      auto* temp = (*type_value).GetIfString();
      if (!temp) {
        out->type = absl::nullopt;
        return false;
      }
      out->type = *temp;
    }
  }

  return true;
}

// static
std::unique_ptr<LaunchItem> LaunchItem::FromValue(const base::Value& value) {
  auto out = std::make_unique<LaunchItem>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict LaunchItem::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("entry", (this->entry).ToValue());

  if (this->type) {
    to_value_result.Set("type", *this->type);

  }

  return to_value_result;
}


const char* ToString(LaunchSource enum_param) {
  switch (enum_param) {
    case LAUNCH_SOURCE_UNTRACKED:
      return "untracked";
    case LAUNCH_SOURCE_APP_LAUNCHER:
      return "app_launcher";
    case LAUNCH_SOURCE_NEW_TAB_PAGE:
      return "new_tab_page";
    case LAUNCH_SOURCE_RELOAD:
      return "reload";
    case LAUNCH_SOURCE_RESTART:
      return "restart";
    case LAUNCH_SOURCE_LOAD_AND_LAUNCH:
      return "load_and_launch";
    case LAUNCH_SOURCE_COMMAND_LINE:
      return "command_line";
    case LAUNCH_SOURCE_FILE_HANDLER:
      return "file_handler";
    case LAUNCH_SOURCE_URL_HANDLER:
      return "url_handler";
    case LAUNCH_SOURCE_SYSTEM_TRAY:
      return "system_tray";
    case LAUNCH_SOURCE_ABOUT_PAGE:
      return "about_page";
    case LAUNCH_SOURCE_KEYBOARD:
      return "keyboard";
    case LAUNCH_SOURCE_EXTENSIONS_PAGE:
      return "extensions_page";
    case LAUNCH_SOURCE_MANAGEMENT_API:
      return "management_api";
    case LAUNCH_SOURCE_EPHEMERAL_APP:
      return "ephemeral_app";
    case LAUNCH_SOURCE_BACKGROUND:
      return "background";
    case LAUNCH_SOURCE_KIOSK:
      return "kiosk";
    case LAUNCH_SOURCE_CHROME_INTERNAL:
      return "chrome_internal";
    case LAUNCH_SOURCE_TEST:
      return "test";
    case LAUNCH_SOURCE_INSTALLED_NOTIFICATION:
      return "installed_notification";
    case LAUNCH_SOURCE_CONTEXT_MENU:
      return "context_menu";
    case LAUNCH_SOURCE_ARC:
      return "arc";
    case LAUNCH_SOURCE_INTENT_URL:
      return "intent_url";
    case LAUNCH_SOURCE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

LaunchSource ParseLaunchSource(const std::string& enum_string) {
  if (enum_string == "untracked")
    return LAUNCH_SOURCE_UNTRACKED;
  if (enum_string == "app_launcher")
    return LAUNCH_SOURCE_APP_LAUNCHER;
  if (enum_string == "new_tab_page")
    return LAUNCH_SOURCE_NEW_TAB_PAGE;
  if (enum_string == "reload")
    return LAUNCH_SOURCE_RELOAD;
  if (enum_string == "restart")
    return LAUNCH_SOURCE_RESTART;
  if (enum_string == "load_and_launch")
    return LAUNCH_SOURCE_LOAD_AND_LAUNCH;
  if (enum_string == "command_line")
    return LAUNCH_SOURCE_COMMAND_LINE;
  if (enum_string == "file_handler")
    return LAUNCH_SOURCE_FILE_HANDLER;
  if (enum_string == "url_handler")
    return LAUNCH_SOURCE_URL_HANDLER;
  if (enum_string == "system_tray")
    return LAUNCH_SOURCE_SYSTEM_TRAY;
  if (enum_string == "about_page")
    return LAUNCH_SOURCE_ABOUT_PAGE;
  if (enum_string == "keyboard")
    return LAUNCH_SOURCE_KEYBOARD;
  if (enum_string == "extensions_page")
    return LAUNCH_SOURCE_EXTENSIONS_PAGE;
  if (enum_string == "management_api")
    return LAUNCH_SOURCE_MANAGEMENT_API;
  if (enum_string == "ephemeral_app")
    return LAUNCH_SOURCE_EPHEMERAL_APP;
  if (enum_string == "background")
    return LAUNCH_SOURCE_BACKGROUND;
  if (enum_string == "kiosk")
    return LAUNCH_SOURCE_KIOSK;
  if (enum_string == "chrome_internal")
    return LAUNCH_SOURCE_CHROME_INTERNAL;
  if (enum_string == "test")
    return LAUNCH_SOURCE_TEST;
  if (enum_string == "installed_notification")
    return LAUNCH_SOURCE_INSTALLED_NOTIFICATION;
  if (enum_string == "context_menu")
    return LAUNCH_SOURCE_CONTEXT_MENU;
  if (enum_string == "arc")
    return LAUNCH_SOURCE_ARC;
  if (enum_string == "intent_url")
    return LAUNCH_SOURCE_INTENT_URL;
  return LAUNCH_SOURCE_NONE;
}


const char* ToString(ActionType enum_param) {
  switch (enum_param) {
    case ACTION_TYPE_NEW_NOTE:
      return "new_note";
    case ACTION_TYPE_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

ActionType ParseActionType(const std::string& enum_string) {
  if (enum_string == "new_note")
    return ACTION_TYPE_NEW_NOTE;
  return ACTION_TYPE_NONE;
}


ActionData::ActionData()
: action_type(ACTION_TYPE_NONE) {}

ActionData::~ActionData() = default;
ActionData::ActionData(ActionData&& rhs) = default;
ActionData& ActionData::operator=(ActionData&& rhs) = default;
// static
bool ActionData::Populate(
    const base::Value& value, ActionData* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* action_type_value = dict.Find("actionType");
  if (!action_type_value) {
    return false;
  }
  {
    const std::string* action_type_as_string = (*action_type_value).GetIfString();
    if (!action_type_as_string) {
      return false;
    }
    out->action_type = ParseActionType(*action_type_as_string);
    if (out->action_type == ACTION_TYPE_NONE) {
      return false;
    }
  }

  const base::Value* is_lock_screen_action_value = dict.Find("isLockScreenAction");
  if (is_lock_screen_action_value) {
    {
      auto temp = (*is_lock_screen_action_value).GetIfBool();
      if (!temp.has_value()) {
        out->is_lock_screen_action = absl::nullopt;
        return false;
      }
      out->is_lock_screen_action = *temp;
    }
  }

  const base::Value* restore_last_action_state_value = dict.Find("restoreLastActionState");
  if (restore_last_action_state_value) {
    {
      auto temp = (*restore_last_action_state_value).GetIfBool();
      if (!temp.has_value()) {
        out->restore_last_action_state = absl::nullopt;
        return false;
      }
      out->restore_last_action_state = *temp;
    }
  }

  return true;
}

// static
std::unique_ptr<ActionData> ActionData::FromValue(const base::Value& value) {
  auto out = std::make_unique<ActionData>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict ActionData::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("actionType", app_runtime::ToString(this->action_type));

  if (this->is_lock_screen_action) {
    to_value_result.Set("isLockScreenAction", *this->is_lock_screen_action);

  }
  if (this->restore_last_action_state) {
    to_value_result.Set("restoreLastActionState", *this->restore_last_action_state);

  }

  return to_value_result;
}


LaunchData::LaunchData()
: source(LAUNCH_SOURCE_NONE) {}

LaunchData::~LaunchData() = default;
LaunchData::LaunchData(LaunchData&& rhs) = default;
LaunchData& LaunchData::operator=(LaunchData&& rhs) = default;
// static
bool LaunchData::Populate(
    const base::Value& value, LaunchData* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  out->source = LAUNCH_SOURCE_NONE;
  const base::Value* id_value = dict.Find("id");
  if (id_value) {
    {
      auto* temp = (*id_value).GetIfString();
      if (!temp) {
        out->id = absl::nullopt;
        return false;
      }
      out->id = *temp;
    }
  }

  const base::Value* items_value = dict.Find("items");
  if (items_value) {
    {
      if (!(*items_value).is_list()) {
        return false;
      }
      else {
        if (!json_schema_compiler::util::PopulateOptionalArrayFromList((*items_value).GetList(), &out->items)) {
          return false;
        }
      }
    }
  }

  const base::Value* url_value = dict.Find("url");
  if (url_value) {
    {
      auto* temp = (*url_value).GetIfString();
      if (!temp) {
        out->url = absl::nullopt;
        return false;
      }
      out->url = *temp;
    }
  }

  const base::Value* referrer_url_value = dict.Find("referrerUrl");
  if (referrer_url_value) {
    {
      auto* temp = (*referrer_url_value).GetIfString();
      if (!temp) {
        out->referrer_url = absl::nullopt;
        return false;
      }
      out->referrer_url = *temp;
    }
  }

  const base::Value* is_demo_session_value = dict.Find("isDemoSession");
  if (is_demo_session_value) {
    {
      auto temp = (*is_demo_session_value).GetIfBool();
      if (!temp.has_value()) {
        out->is_demo_session = absl::nullopt;
        return false;
      }
      out->is_demo_session = *temp;
    }
  }

  const base::Value* is_kiosk_session_value = dict.Find("isKioskSession");
  if (is_kiosk_session_value) {
    {
      auto temp = (*is_kiosk_session_value).GetIfBool();
      if (!temp.has_value()) {
        out->is_kiosk_session = absl::nullopt;
        return false;
      }
      out->is_kiosk_session = *temp;
    }
  }

  const base::Value* is_public_session_value = dict.Find("isPublicSession");
  if (is_public_session_value) {
    {
      auto temp = (*is_public_session_value).GetIfBool();
      if (!temp.has_value()) {
        out->is_public_session = absl::nullopt;
        return false;
      }
      out->is_public_session = *temp;
    }
  }

  const base::Value* source_value = dict.Find("source");
  if (source_value) {
    {
      const std::string* launch_source_as_string = (*source_value).GetIfString();
      if (!launch_source_as_string) {
        return false;
      }
      out->source = ParseLaunchSource(*launch_source_as_string);
      if (out->source == LAUNCH_SOURCE_NONE) {
        return false;
      }
    }
    } else {
    out->source = LAUNCH_SOURCE_NONE;
  }

  const base::Value* action_data_value = dict.Find("actionData");
  if (action_data_value) {
    {
      if (!(*action_data_value).is_dict()) {
        return false;
      }
      else {
        ActionData temp;
        if (!ActionData::Populate((*action_data_value), &temp))
          return false;
        out->action_data = std::move(temp);
      }
    }
  }

  return true;
}

// static
std::unique_ptr<LaunchData> LaunchData::FromValue(const base::Value& value) {
  auto out = std::make_unique<LaunchData>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict LaunchData::ToValue() const {
  base::Value::Dict to_value_result;

  if (this->id) {
    to_value_result.Set("id", *this->id);

  }
  if (this->items) {
    to_value_result.Set("items", json_schema_compiler::util::CreateValueFromArray(*this->items));

  }
  if (this->url) {
    to_value_result.Set("url", *this->url);

  }
  if (this->referrer_url) {
    to_value_result.Set("referrerUrl", *this->referrer_url);

  }
  if (this->is_demo_session) {
    to_value_result.Set("isDemoSession", *this->is_demo_session);

  }
  if (this->is_kiosk_session) {
    to_value_result.Set("isKioskSession", *this->is_kiosk_session);

  }
  if (this->is_public_session) {
    to_value_result.Set("isPublicSession", *this->is_public_session);

  }
  if (this->source != LAUNCH_SOURCE_NONE) {
    to_value_result.Set("source", app_runtime::ToString(this->source));

  }
  if (this->action_data) {
    to_value_result.Set("actionData", (this->action_data)->ToValue());

  }

  return to_value_result;
}


EmbedRequest::EmbedRequest()
 {}

EmbedRequest::~EmbedRequest() = default;
EmbedRequest::EmbedRequest(EmbedRequest&& rhs) = default;
EmbedRequest& EmbedRequest::operator=(EmbedRequest&& rhs) = default;
// static
bool EmbedRequest::Populate(
    const base::Value& value, EmbedRequest* out) {
  if (!value.is_dict()) {
    return false;
  }
  const base::Value::Dict& dict = value.GetDict();
  const base::Value* embedder_id_value = dict.Find("embedderId");
  if (!embedder_id_value) {
    return false;
  }
  {
    auto* temp = (*embedder_id_value).GetIfString();
    if (!temp) {
      return false;
    }
    out->embedder_id = *temp;
  }

  const base::Value* data_value = dict.Find("data");
  if (data_value) {
    {
      out->data = (*data_value).Clone();
    }
  }

  return true;
}

// static
std::unique_ptr<EmbedRequest> EmbedRequest::FromValue(const base::Value& value) {
  auto out = std::make_unique<EmbedRequest>();
  bool result = Populate(value, out.get());
  if (!result)
    return nullptr;
  return out;
}

base::Value::Dict EmbedRequest::ToValue() const {
  base::Value::Dict to_value_result;

  to_value_result.Set("embedderId", this->embedder_id);

  if (this->data) {
    to_value_result.Set("data", (this->data)->Clone());

  }

  return to_value_result;
}



//
// Events
//

namespace OnEmbedRequested {

const char kEventName[] = "app.runtime.onEmbedRequested";

base::Value::List Create(const EmbedRequest& request) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((request).ToValue());

  return create_results;
}

}  // namespace OnEmbedRequested

namespace OnLaunched {

const char kEventName[] = "app.runtime.onLaunched";

base::Value::List Create(const LaunchData& launch_data) {
  base::Value::List create_results;
  create_results.reserve(1);
  create_results.Append((launch_data).ToValue());

  return create_results;
}

}  // namespace OnLaunched

namespace OnRestarted {

const char kEventName[] = "app.runtime.onRestarted";

base::Value::List Create() {
  base::Value::List create_results;

  return create_results;
}

}  // namespace OnRestarted

}  // namespace app_runtime
}  // namespace api
}  // namespace extensions

